"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isBuffer = require("is-buffer");
var Debugger = require("debug");
var stream_1 = require("./stream");
var debug = Debugger('akita:result');
var Result = (function () {
    function Result(fetch, path, init, query, reducer) {
        this._query = query;
        this._reducer = reducer;
        this._path = path;
        this._init = init;
        var promise = fetch(path, init);
        if (debug.enabled) {
            promise = promise.then(function (res) {
                debug('response status:', res.status, res.statusText);
                return res;
            }, function (error) {
                debug('fetch error:', error.message);
                return Promise.reject(error);
            });
        }
        this._responsePromise = promise;
    }
    Result.prototype.response = function () {
        return this._responsePromise;
    };
    Result.prototype.stream = function () {
        return this.response().then(function (res) { return res.body; });
    };
    Result.prototype.ok = function () {
        return this.response().then(function (res) { return res.ok; });
    };
    Result.prototype.status = function () {
        return this.response().then(function (res) { return res.status; });
    };
    Result.prototype.statusText = function () {
        return this.response().then(function (res) { return res.statusText; });
    };
    Result.prototype.size = function () {
        return this.response().then(function (res) { return res.size || parseInt(res.headers.get('Content-Length')) || 0; });
    };
    Result.prototype.headers = function () {
        return this.response().then(function (res) { return res.headers; });
    };
    Result.prototype.buffer = function () {
        return this.response().then(function (res) {
            var fn = 'buffer';
            if (res.arrayBuffer) {
                fn = 'arrayBuffer';
            }
            return res[fn]().then(function (buf) {
                if (!isBuffer(buf) && buf instanceof ArrayBuffer && typeof Buffer === 'function') {
                    return Buffer.from(buf);
                }
                return buf;
            }, function (error) {
                debug('get buffer error:', error.message);
                return Promise.reject(error);
            });
        });
    };
    Result.prototype.blob = function () {
        return this.response().then(function (res) { return res.blob(); });
    };
    Result.prototype.text = function () {
        return this.response().then(function (res) { return res.text().then(function (text) {
            debug('response text:', text);
            return text;
        }); });
    };
    Result.prototype.json = function () {
        var _this = this;
        return this.response().then(function (res) {
            if (res.status === 204) {
                return Promise.resolve();
            }
            return res.text().then(function (text) {
                debug('response text:', text);
                if (res.status === 404 && _this._query && ['findByPk', 'remove'].indexOf(_this._query._op) > -1) {
                    debug("return null when " + _this._query._op + " 404");
                    return null;
                }
                var json;
                try {
                    json = JSON.parse(text);
                }
                catch (error) {
                    return Promise.reject(new Error("invalid json response body at " + _this._path + " " + error.message));
                }
                if (json && json.error && ['0', 'null', 'none'].indexOf(json.error) < 0) {
                    var error_1 = new Error(json.error);
                    Object.keys(json).forEach(function (key) {
                        if (['error', 'message', 'stack'].indexOf(key) === -1) {
                            error_1[key] = json[key];
                        }
                    });
                    return Promise.reject(error_1);
                }
                return json;
            }, function (error) {
                debug('json parse error:', error.message);
                return Promise.reject(error);
            });
        });
    };
    Result.prototype.then = function (onSuccess, onFail) {
        var _this = this;
        if (this._query && this._query._op === 'watch') {
            if (!this._csPromise) {
                this._csPromise = this.stream().then(function (stream) { return new stream_1.default(stream, _this._reducer); });
            }
            return this._csPromise.then(onSuccess, onFail);
        }
        if (this._reducer) {
            return this.json().then(function (json) { return onSuccess(_this._reducer(json)); }, onFail);
        }
        return this.json().then(onSuccess, onFail);
    };
    Result.prototype.catch = function (onFail) {
        return this.json().catch(onFail);
    };
    return Result;
}());
exports.default = Result;
